/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import com.sas.drugdev.remote.RemoteException;
import com.sas.drugdev.remote.ServiceManager;
import com.sas.drugdev.remote.repository.AclRecurseBehavior;
import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.FileBean;
import com.sas.drugdev.remote.repository.InvalidNodeBeanException;
import com.sas.drugdev.remote.repository.InvalidNodeException;
import com.sas.drugdev.remote.repository.InvalidTypeException;
import com.sas.drugdev.remote.repository.InvalidVersionException;
import com.sas.drugdev.remote.repository.Node;
import com.sas.drugdev.remote.repository.NodeBean;
import com.sas.drugdev.remote.repository.NodeId;
import com.sas.drugdev.remote.repository.NodeModifiedException;
import com.sas.drugdev.remote.repository.PermissionException;
import com.sas.drugdev.remote.repository.RemoteContainer;
import com.sas.drugdev.remote.repository.RemoteFile;
import com.sas.drugdev.remote.repository.RemoteNode;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.repository.ValidationException;
import com.sas.drugdev.remote.repository.security.AccessControlEntry;
import com.sas.drugdev.remote.repository.security.AccessControlList;
import com.sas.drugdev.remote.repository.security.Permission;
import com.sas.drugdev.remote.session.AuthenticationException;
import com.sas.drugdev.remote.session.Credentials;
import com.sas.drugdev.remote.session.InvalidSessionException;
import com.sas.drugdev.remote.session.PasswordExpiredException;
import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.session.SessionFactory;
import com.sas.drugdev.remote.session.UnsupportedCredentialsException;
import com.sas.drugdev.remote.session.UserInactiveException;
import com.sas.drugdev.remote.session.UserRetiredException;
import com.sas.drugdev.remote.session.UsernamePasswordCredentials;
import com.sas.drugdev.remote.type.Type;
import com.sas.drugdev.remote.type.TypeBean;
import com.sas.drugdev.remote.type.TypeService;


public class SASDrugDevRemoteAPIRepositoryServiceSample 
{
    private static final int COMMAND_GET_CHILDREN=1;
    private static final int COMMAND_GET_TYPES=2;
    private static final int COMMAND_CREATE_NODE=3;
    private static final int COMMAND_GET_FILE=4;
    private static final int COMMAND_GET_FILE_PROPS=5;
    private static final int COMMAND_GET_CONTAINER_PROPS=6;
    private static final int COMMAND_SET_FILE_PROPS=7;
    private static final int COMMAND_SET_CONTAINER_PROPS=8;
    private static final int COMMAND_COPY_FILE=9;
    private static final int COMMAND_COPY_CONTAINER=10;
    private static final int COMMAND_MOVE_FILE=11;
    private static final int COMMAND_MOVE_CONTAINER=12;
    private static final int COMMAND_DELETE_FILE=13;
    private static final int COMMAND_DELETE_CONTAINER=14;
    private static final int COMMAND_ENABLE_VERSIONING=15;
    private static final int COMMAND_CHECKOUT_CHECKIN=16;
    private static final int COMMAND_ADD_FILE_ACE=17;
    private static final int COMMAND_CHANGE_FILE_PERMS=18;
    private static final int COMMAND_CHANGE_CONTAINER_PERMS=19;
    private static final int COMMAND_TEST_SESSION=20;
    private static final int COMMAND_EXIT=21;

    private Session session;
    private ServiceManager serviceMgr;
    private RepositoryService repositoryService;

    public SASDrugDevRemoteAPIRepositoryServiceSample(URL serverURL, String userName, String password)
    {
        init(serverURL, userName, password);
    }

    // Initialize the SASDrugDev Remote API client
    private void init(URL serverURL, String userName, String password)
    {
        // Create a new session
        session = createSASDrugDevSession(serverURL, userName, password);

        // Check for a valid session and initialize service manager
        if ( session.isValid())
        {
            serviceMgr = session.getServiceManager();  
            repositoryService = serviceMgr.getRepositoryService();
        }
    }

    // Checks if the current connection to to the SAS Drug Dev server is valid. 
    public boolean isConnected() {
        return session.isValid();
    } 

    // Create session/open connection to SDD server
    public Session createSASDrugDevSession(URL serverURL, String userName, String password)
    {
        // Create a session using the given user credentials
        Session session = null;
        try {

            Credentials credentials = new UsernamePasswordCredentials(userName, password.toCharArray());
            session = SessionFactory.newSession(serverURL, credentials);                

        } catch (AuthenticationException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PasswordExpiredException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UserInactiveException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UserRetiredException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (UnsupportedCredentialsException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
        return session;
    }

    // Close connection to SDD
    public void closeSASDrugDevSession()
    {
        // check if session is valid before closing server session
        if ( session.isValid())
        {
            // invalidate session and free up server side resources held by this session
            session.invalidate();
        }  
    }

    public void getFileProperties(String path) {
        getNodeProperties(new FileBean(path));
    }

    public void getContainerProperties(String path) {
        getNodeProperties(new ContainerBean(path));
    }

    // Sample code to get properties
    public void getNodeProperties(NodeBean nodeBean)
    {
        try {
            RemoteNode node = repositoryService.getNode(nodeBean);

            if ( node != null ) {
                System.out.println("\nGetting properties for "+ node.getPath());
                Map properties = node.getProperties();
    
                // lookup properties by name
    
                // Iterate through properties
                for (Iterator iter = properties.keySet().iterator(); iter.hasNext();) {
    
                    String propertyName = (String) iter.next();
                    String propertyValue = (String)properties.get(propertyName);
    
                    // do something with property, for ex: print them out
                    System.out.println(propertyName + " = " + propertyValue);
                }
            } else {
                System.out.println("Node doesn't exist: " + nodeBean.getPath());
            }

        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Example to show how node properties can be modified individually
    public void setNodeProperties(NodeBean updateBean, String propertyName, String propertyValue)
    {
        try {
            RemoteNode node = repositoryService.getNode(updateBean);

            // Checking write permission will require a call to the server
            if ( node.canWrite())
            {
                // set single property
                updateBean.setProperty(propertyName, propertyValue);                  

                node = repositoryService.updateNode(updateBean);
                System.out.println("Modified " + propertyName + " to " + propertyValue.toString());    
            }
            else
            {
                System.out.println("You do not have permissions to modify this node");
            }

        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeException e) {
            // Handle exception
            System.err.println(e.getMessage());            
        } catch (InvalidNodeBeanException e) {
            // Handle exception
            System.err.println(e.getMessage());   
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (NodeModifiedException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (ValidationException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidTypeException e) {
            // Handle exception
            System.err.println(e.getMessage());            
        } catch (PermissionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }



    // Create a new file of the specified type in SDD
    public NodeId createFileInSDD(String path, String typeName, String localFilePath, Properties properties)
    {
        NodeId nodeId = null;
        try {

            // Construct a FileBean with the SDD path and specified file type
            FileBean fileBean = new FileBean(path, new TypeBean(typeName));

            // Set Contents of the FileBean using the localFile to be pushed to SAS Drug Dev repository
            fileBean.setContents(new File(localFilePath));

            // Set properties if available, else will be set to default values
            fileBean.setProperties(properties);

            // Create this file on the server
            RemoteFile remoteFile = repositoryService.createFile(fileBean);

            // Create successful, use remoteFile to perform other actions on this file. 
            System.out.println("\n Created new file" + remoteFile.getPath() + " with id = " + remoteFile.getId() + "\n");               

        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

        return nodeId;
    }

    // Get a file of specified path from SDD. If file id is available, that could be used instead of the file path
    public void getFile(String sddFilePath)
    {
        try {

            FileBean fileBean = new FileBean(sddFilePath);
            RemoteFile remoteFile = repositoryService.getFile(fileBean);
            System.out.println();
            System.out.println("Got File: " + remoteFile.getPath() + " - ID = " + remoteFile.getId());
        } catch (IllegalStateException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidVersionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidTypeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // List the different node/object types available in SAS Drug Development domain
    public void listSDDNodeTypes()
    {
        try {

            TypeService typeService = serviceMgr.getTypeService();

            if ( typeService != null)
            {
                List sddTypes = typeService.getTypes();

                if ( sddTypes != null)
                {
                    System.out.println("\n Node Types defined on SAS Drug Dev Server : \n");
                    for (Iterator iter = sddTypes.iterator(); iter.hasNext();) {
                        Type type = (Type)iter.next();
                        System.out.println("type name =" + type.getName());
                    }
                }
            }

        } catch (RemoteException e) {
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            System.err.println(e.getMessage());
        }

    }

    // Copy a file or container from one location in SDD to another
    public void copyFileOrContainer(NodeBean srcBean, ContainerBean destBean)
    {
        RemoteNode copiedNode = null;
        try {
            copiedNode = repositoryService.copyNode(srcBean, destBean);
            String newPath = copiedNode.getPath();

            System.out.println();
            System.out.println(srcBean.getPath() + " has been copied to " + newPath  );
        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Move a file or container from one location in SDD to another
    public void moveFileOrContainer(NodeBean srcBean, ContainerBean destBean)
    {
        try {

            RemoteNode moveNode = repositoryService.moveNode(srcBean, destBean);
            String newPath = moveNode.getPath();

            System.out.println();
            System.out.println(srcBean.getPath() + " has been moved to " + newPath  );

        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Delete a file or container in SDD
    public void deleteNode(NodeBean nodeBean)
    {

        try {

            // Delete the given object in SDD using the repository service
            repositoryService.deleteNode(nodeBean);

            System.out.println();
            System.out.println("Successfully deleted "+ nodeBean.getPath());

        } catch (IllegalStateException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PermissionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }

    // List contents of a directory
    public void getChildren(String sddContainerPath)
    {
        try {

            // Get children for the specified container using repository service
            List children = repositoryService.getChildren(new ContainerBean(sddContainerPath));

            // Print the child node paths
            System.out.println("\n Children of "+ sddContainerPath + " are:\n");

            for (Iterator iter = children.iterator(); iter.hasNext();) {
                Node node = (Node) iter.next();
                System.out.println(node.getPath());
            }

        } catch (IllegalStateException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidTypeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Enable versioning on a file
    public void enableFileVersioning(String sddFilePath)
    {
        try {

            // Construct FileBean class using the SAS Drug Dev file path
            FileBean sddFileBean = new FileBean(sddFilePath);

            // Turn versioning on using repository service
            RemoteFile versionedFile = repositoryService.enableVersioning(sddFileBean);

            System.out.println("\n Versioning is enabled on " + versionedFile.getPath());

        } catch (InvalidTypeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PermissionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        }
    }

    // Checkin/Checkout a file and print file version information
    public void checkOutCheckIn(String path, String checkinFilePath)
    {

        try {

            // Construct a FileBean with given path
            FileBean fileBean = new FileBean(path);

            // Checkout the file using Repository Service
            RemoteFile remoteFile = repositoryService.checkoutFile(fileBean);

            // Construct a filebean for the version to be checked in
            FileBean checkinFileBean =  new FileBean(remoteFile);

            // Set contents of new version 
            File localFile = new File(checkinFilePath);
            checkinFileBean.setContents(localFile);

            // Update the working version using the Repository Service
            RemoteFile updatedFile = (RemoteFile) repositoryService.updateNode(checkinFileBean);
            RemoteFile checkedinFile = repositoryService.checkinFile(updatedFile);

            
            // Get version information
            List versions = checkedinFile.getVersions();

            if ( versions != null)
            {
                System.out.println("\n File " + path + " has " + versions.size() + " versions");

                for ( Iterator iter = versions.iterator(); iter.hasNext();)
                {                        
                    String version = (String)iter.next();

                    // Print version information
                    System.out.println(version);
                }
            }

        } catch (IllegalStateException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (PermissionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidTypeException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (RemoteException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidSessionException e) {
            // Handle exception
            System.err.println(e.getMessage());
        } catch (InvalidNodeBeanException e) {
            System.err.println(e.getMessage());
        } catch (NodeModifiedException e) {
            System.err.println(e.getMessage());
        } catch (ValidationException e) {
            System.err.println(e.getMessage());
        }

    }


    // Example to remove and add a group/user from permissions for an object
    public void manageFilePermissions(String sddFilePath, String removeGroupName, String removeUserId, String addUserId, String addGroupName )
    {

        try {
            FileBean fileBean = new FileBean(sddFilePath);
            RemoteFile remoteFile = repositoryService.getFile(fileBean);


            // Get access control list for node
            AccessControlList acl = repositoryService.getAccessControlList(remoteFile);

            // Get and print the current access control entries that are defined for this node
            List currentAceEntries = acl.getEntries();
            System.out.println("\nCurrent Access Control List for " + sddFilePath + ": \n");

            for (Iterator iter = currentAceEntries.iterator(); iter.hasNext();) {
                AccessControlEntry ace = (AccessControlEntry) iter.next();

                if (ace.isGroupEntry())
                {
                    System.out.println("Group Entry ACE, name =" + ace.getName());
                }
                else
                {
                    System.out.println("User Entry ACE, name =" + ace.getName());
                }

            }    

            // Remove group from Access Control List
            if (! "".equals(removeGroupName)) {                    
                // remove entry corresponding to specified group name
                acl.removeGroupEntry(removeGroupName);
            }

            // Remove userid from Access Control List
            if (! "".equals(removeGroupName)) {                    
                // remove entry corresponding to userid to be removed from ACL
                acl.removeUserEntry(removeUserId);
            }

            // Construct permissions to be used for the user/group to be added
            Set permissions = new HashSet();
            permissions.add(Permission.DELETE_PERMISSION);
            permissions.add(Permission.READ_PERMISSION);
            permissions.add(Permission.WRITE_PERMISSION);
            permissions.add(Permission.MANAGE_PERMISSION);

            // Add userid to Access Control List
            if (! "".equals(addUserId)) {                    

                // Create entry for user to be added
                AccessControlEntry newUserAce = acl.createUserEntry(addUserId);
                newUserAce.setPermissions(permissions);
                acl.addEntry(newUserAce); 
            }

            // Add group to Access Control List
            if ( ! "".equals(addGroupName))
            {
                AccessControlEntry newGroupAce = acl.createGroupEntry(addGroupName);
                newGroupAce.setPermissions(permissions);
                acl.addEntry(newGroupAce); 
            }

            // Perform the ACL changes on the server
            repositoryService.updateFileAccessControlList(acl);

            // Retrieve Access Control List that was updated and print 
            AccessControlList updatedAcl = repositoryService.getAccessControlList(remoteFile);
            List aceEntries = updatedAcl.getEntries();                    

            System.out.println("\nUpdated Access Control List for " + sddFilePath + ": \n");

            for (Iterator iter = aceEntries.iterator(); iter.hasNext();) {
                AccessControlEntry ace = (AccessControlEntry) iter.next();

                if (ace.isGroupEntry())
                {
                    System.out.println("Group Entry ACE, name =" + ace.getName());
                }
                else
                {
                    System.out.println("User Entry ACE, name =" + ace.getName());
                }

            }                    

        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }                                               

    }


    // Get Access Control List of a file and grant delete and write permissions
    public void changeFilePermissions(String sddFilePath)
    {

        try {
            FileBean fileBean = new FileBean(sddFilePath);
            RemoteFile remoteFile = repositoryService.getFile(fileBean);

            // Set write and delete permissions for current user
            AccessControlList acl = repositoryService.getAccessControlList(remoteFile);
            List aceEntries = acl.getEntries();

            AccessControlEntry ace;
            for (Iterator iter = aceEntries.iterator(); iter.hasNext();) {
                ace = (AccessControlEntry) iter.next();
                if ( ! ace.hasPermission(Permission.DELETE_PERMISSION))
                {
                    ace.addPermission(Permission.DELETE_PERMISSION);
                }
                if ( ! ace.hasPermission(Permission.WRITE_PERMISSION))
                {
                    ace.addPermission(Permission.WRITE_PERMISSION);
                }
            }


            repositoryService.updateFileAccessControlList(acl);

            System.out.println("\nAccess Control List Updated, delete and write permissions granted to all access control entries");

        } catch (Exception  e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }

    // Reset permissions of a container node
    public void changeContainerPermissions(String sddContainerPath)
    {

        try {
            ContainerBean containerBean = new ContainerBean(sddContainerPath);
            RemoteContainer remoteContainer = repositoryService.getContainer(containerBean);

            // Set write and delete permissions for current user
            AccessControlList acl = repositoryService.getAccessControlList(remoteContainer);
            List aceEntries = acl.getEntries();

            Set permissions = new HashSet();
            permissions.add(Permission.DELETE_PERMISSION);
            permissions.add(Permission.READ_PERMISSION);
            permissions.add(Permission.WRITE_PERMISSION);
            permissions.add(Permission.MANAGE_PERMISSION);
            permissions.add(Permission.INHERIT_READ_PERMISSION);
            permissions.add(Permission.INHERIT_DELETE_PERMISSION);
            permissions.add(Permission.INHERIT_WRITE_PERMISSION);

            AccessControlEntry ace;
            for (Iterator iter = aceEntries.iterator(); iter.hasNext();) {
                ace = (AccessControlEntry) iter.next();
                ace.setPermissions(permissions);
            }

            repositoryService.updateContainerAccessControlList(acl, AclRecurseBehavior.RECURSE_NONE);
            System.out.println("\nAccess Control List Updated, delete and write permissions granted to all access control entries of container");

        } catch (Exception e) {
            // Handle exception
            System.err.println(e.getMessage());
        }

    }

    public static void main(String[] argv)
    {
        URL serverURL = null;
        String serverURLString = null;
        String userid = null;
        String password = null;         


        if (argv.length < 1) {
            showUsage();
            System.exit(1);
        }

        for (int i = 0; i < argv.length; i++) {
            if (argv[i].equalsIgnoreCase("-s")) {
                i++;
                if (i < argv.length) serverURLString = argv[i];
            } else if (argv[i].equalsIgnoreCase("-u")) {
                i++;
                if (i < argv.length) userid = argv[i];
            } else if (argv[i].equalsIgnoreCase("-p")) {
                i++;
                if (i < argv.length) password = argv[i];
            } 
        }


        if (serverURLString == null) {
            serverURLString = String.valueOf(readInput("Server URL: "));
        }
        try {
            serverURL = new URL(serverURLString);
        } catch (MalformedURLException me) {
            System.err.println("The server URL is not valid.");
            showUsage();
            System.exit(1);
        }
        if (userid == null || userid.length() == 0) {
            userid = String.valueOf(readInput("Username: "));
        }
        if (password == null || String.valueOf(password).length() == 0) {
            password = readInput("Password: ");
        }

        SASDrugDevRemoteAPIRepositoryServiceSample sddRemoteAPIClient = new SASDrugDevRemoteAPIRepositoryServiceSample(serverURL, userid, password);
        if ( sddRemoteAPIClient.isConnected() )
        {
            System.out.println("Connected to the server..\n");
        }
        else
        {
            System.out.println("Error connecting to server");
            System.exit(1);
        }


        boolean exitSample = false;

        try
        {
            while ( ! exitSample)
            {
                try { 
                    System.out.println("\n\nPlease select a command to execute: ");
                    showCommands();
                    System.out.println("\n");
                    int commandName = Integer.valueOf(readInput("Command : ")).intValue();

                    switch (commandName) {
                    case COMMAND_GET_CHILDREN:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Container Path : ")); 
                        sddRemoteAPIClient.getChildren(path);
                        break;
                    }
                    case COMMAND_GET_TYPES:
                        sddRemoteAPIClient.listSDDNodeTypes();
                        break;
                    case COMMAND_CREATE_NODE:
                    {
                        String sddFilePath = String.valueOf(readInput("SAS Drug Dev Path : "));
                        String typeName = String.valueOf(readInput("SAS Drug Dev type name : "));
                        String localFilePath = String.valueOf(readInput("Local File Path : "));

                        Properties properties = new Properties();                        
                        properties.put("description", "Created by sample code");

                        sddRemoteAPIClient.createFileInSDD(sddFilePath, typeName, localFilePath, properties );                            
                        break;
                    }
                    case COMMAND_GET_FILE:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev File Path : "));
                        sddRemoteAPIClient.getFile(path);
                        break;
                    }
                    case COMMAND_GET_FILE_PROPS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path : "));
                        sddRemoteAPIClient.getFileProperties(path);
                        break;
                    }
                    case COMMAND_GET_CONTAINER_PROPS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path : "));
                        sddRemoteAPIClient.getContainerProperties(path);
                        break;
                    }
                    case COMMAND_SET_FILE_PROPS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path : "));
                        String propertyName = String.valueOf(readInput("Name of node property to be changed : "));
                        String propertyValue = String.valueOf(readInput("New value : "));
                        sddRemoteAPIClient.setNodeProperties(new FileBean(path), propertyName, propertyValue);
                        break;
                    }
                    case COMMAND_SET_CONTAINER_PROPS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path : "));
                        String propertyName = String.valueOf(readInput("Name of node property to be changed : "));
                        String propertyValue = String.valueOf(readInput("New value : "));
                        sddRemoteAPIClient.setNodeProperties(new ContainerBean(path), propertyName, propertyValue);
                        break;
                    }
                    case COMMAND_COPY_FILE:
                    {
                        String srcSDDPath = String.valueOf(readInput("SAS Drug Dev source path : "));
                        String destSDDPath = String.valueOf(readInput("SAS Drug Dev destination path  : "));
                        sddRemoteAPIClient.copyFileOrContainer(new FileBean(srcSDDPath), new ContainerBean(destSDDPath));
                        break;
                    }
                    case COMMAND_COPY_CONTAINER:
                    {
                        String srcSDDPath = String.valueOf(readInput("SAS Drug Dev source path : "));
                        String destSDDPath = String.valueOf(readInput("SAS Drug Dev destination path  : "));
                        sddRemoteAPIClient.copyFileOrContainer(new ContainerBean(srcSDDPath), new ContainerBean(destSDDPath));
                        break;
                    }
                    case COMMAND_MOVE_FILE:
                    {
                        String srcSDDPath = String.valueOf(readInput("SAS Drug Dev source path : "));
                        String destSDDPath = String.valueOf(readInput("SAS Drug Dev destination path  : "));
                        sddRemoteAPIClient.moveFileOrContainer(new FileBean(srcSDDPath), new ContainerBean(destSDDPath));                      
                        break;
                    }
                    case COMMAND_MOVE_CONTAINER:
                    {
                        String srcSDDPath = String.valueOf(readInput("SAS Drug Dev source path : "));
                        String destSDDPath = String.valueOf(readInput("SAS Drug Dev destination path  : "));
                        sddRemoteAPIClient.moveFileOrContainer(new ContainerBean(srcSDDPath), new ContainerBean(destSDDPath));                      
                        break;
                    }
                    case COMMAND_DELETE_FILE:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path of file to be deleted : "));
                        sddRemoteAPIClient.deleteNode(new FileBean(path));
                        break;
                    }
                    case COMMAND_DELETE_CONTAINER:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Path of container to be deleted : "));
                        sddRemoteAPIClient.deleteNode(new ContainerBean(path));
                        break;
                    }
                    case COMMAND_ENABLE_VERSIONING:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev File Path : "));                            
                        sddRemoteAPIClient.enableFileVersioning(path);
                        break;
                    }
                    case COMMAND_CHECKOUT_CHECKIN:
                    {
                        String path = readInput("SAS Drug Dev Versioned File Path : ");
                        String localFilePath = readInput("Local file path to be checked in : ");
                        sddRemoteAPIClient.checkOutCheckIn(path, localFilePath);
                        break;
                    }
                    case COMMAND_ADD_FILE_ACE:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev File Path : "));
                        String removeGroupName = String.valueOf(readInput("Group Name to be removed : "));
                        if ( removeGroupName == null)
                        {
                            removeGroupName = "";
                        }
                        String removeUserId = String.valueOf(readInput("UserId to be removed : "));
                        String addGroupName = String.valueOf(readInput("Group Name to be added : "));
                        String addUserId = String.valueOf(readInput("UserId to be added : "));

                        if ( removeUserId == null)
                        {
                            removeUserId = "";
                        }

                        if ( addUserId == null)
                        {
                            addUserId = "";
                        }

                        if ( addGroupName == null)
                        {
                            addGroupName = "";
                        }

                        sddRemoteAPIClient.manageFilePermissions(path, removeGroupName, removeUserId, addUserId, addGroupName);
                        break;
                    }
                    case COMMAND_CHANGE_FILE_PERMS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev File Path : "));                      
                        sddRemoteAPIClient.changeFilePermissions(path);
                        break;
                    }
                    case COMMAND_CHANGE_CONTAINER_PERMS:
                    {
                        String path = String.valueOf(readInput("SAS Drug Dev Container Path : "));                      
                        sddRemoteAPIClient.changeContainerPermissions(path);
                        break;
                    }
                    case COMMAND_TEST_SESSION:
                    {
                        if (sddRemoteAPIClient.isConnected()) {
                            System.out.println("Connected to the server!");
                        } else {
                            System.out.println();
                            System.out.println("**Connection to " + serverURL + " is not valid.**");
                            System.out.println();
                        }
                        break;
                    }
                    case COMMAND_EXIT:
                    {
                        exitSample = true;
                        break;
                    }
                    default:
                    {
                        System.err.println("\n An Invalid Command was entered! \n");                        
                        System.out.println("Available commands are:\n");
                        showCommands();
                        break;
                    }
                    }
                } catch (Exception e) {
                    System.out.println("A failure occured while processing this command, error message is :");
                    System.out.println(e.getMessage());
                    e.printStackTrace();

                }
            }

        }
        finally
        {
            sddRemoteAPIClient.closeSASDrugDevSession();
        }

        System.exit(0);
    }

    private static void showUsage()
    {
        System.out.println("Usage: SASDrugDevRemoteAPIAdminSamples -s <server URL> [-u <username>] [-p <password>]");            
    }

    private static void showCommands()
    {            
        System.out.println("1 : List children of a Container");
        System.out.println("2 : List available SAS Drug Dev Node Types");
        System.out.println("3 : Create a node of specified type");
        System.out.println("4 : Get an existing file");            
        System.out.println("5 : Get properties on a File");
        System.out.println("6 : Get properties on a Container");
        System.out.println("7 : Set Properties on a File");
        System.out.println("8 : Set Properties on a Container");
        System.out.println("9 : Copy File");
        System.out.println("10 : Copy Container");
        System.out.println("11 : Move File");          
        System.out.println("12 : Move Container");          
        System.out.println("13 : Delete a File");
        System.out.println("14 : Delete a Container");
        System.out.println("15 : Enable Versioning on a File");
        System.out.println("16 : Checkout and Checkin a File");
        System.out.println("17 : Add a User/Group ACE to a File");
        System.out.println("18 : Change File Permissions");
        System.out.println("19 : Change Container Permissions");            
        System.out.println("20 : Check if connection to server is valid");
        System.out.println("21 : Exit the Remote API Sample Program");
    }

    private static String readInput(String prompt)
    {
        char[] ret = new char[] {'\0'};
        try {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            System.out.print(prompt);
            ret = (in.readLine()).toCharArray();
        } catch (IOException e) {
            System.err.println("Error reading input.");
        }
        return new String(ret);
    }



}
