/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2007, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import com.sas.drugdev.remote.RemoteException;
import com.sas.drugdev.remote.repository.ContainerBean;
import com.sas.drugdev.remote.repository.InvalidNodeException;
import com.sas.drugdev.remote.repository.InvalidTypeException;
import com.sas.drugdev.remote.repository.Node;
import com.sas.drugdev.remote.repository.RepositoryService;
import com.sas.drugdev.remote.session.AuthenticationException;
import com.sas.drugdev.remote.session.InvalidSessionException;
import com.sas.drugdev.remote.session.PasswordExpiredException;
import com.sas.drugdev.remote.session.Session;
import com.sas.drugdev.remote.session.SessionFactory;
import com.sas.drugdev.remote.session.UnsupportedCredentialsException;
import com.sas.drugdev.remote.session.UserInactiveException;
import com.sas.drugdev.remote.session.UsernamePasswordCredentials;
import com.sas.drugdev.remote.session.UserRetiredException;

public class SASDrugDevRemoteAPISample
{
	public static void main(String[] argv)
	{
		URL serverURL = null;
		String serverURLString = null;
		String userid = null;
		char[] password = null;
		int command = -1;
		
		String listPath = null;
                                   
		if (argv.length < 1) {
			showUsage();
			System.exit(1);
		}

		for (int i = 0; i < argv.length; i++) {
			if (argv[i].equalsIgnoreCase("-s")) {
				i++;
				if (i < argv.length) serverURLString = argv[i];
			} else if (argv[i].equalsIgnoreCase("-u")) {
				i++;
				if (i < argv.length) userid = argv[i];
			} else if (argv[i].equalsIgnoreCase("-p")) {
				i++;
				if (i < argv.length) password = argv[i].toCharArray();
			} else if (argv[i].equalsIgnoreCase("-test")) {
				command = 0;
			} else if (argv[i].equalsIgnoreCase("-list")) {
				command = 1;
				i++;
				if (i < argv.length) listPath = argv[i];
			}
		}
		
		if (command < 0) {
			System.err.println("No command specified.");
			showUsage();
			System.exit(1);
		}

		if (serverURLString == null) {
			serverURLString = String.valueOf(readInput("Server URL: "));
		}
		try {
			serverURL = new URL(serverURLString);
		} catch (MalformedURLException me) {
			System.err.println("The server URL is not valid.");
			showUsage();
			System.exit(1);
		}
		if (userid == null || userid.length() == 0) {
			userid = String.valueOf(readInput("Username: "));
		}
		if (password == null || String.valueOf(password).length() == 0) {
			password = readInput("Password: ");
		}

		Session session = null;
		RepositoryService repositoryService = null;
		boolean connectionSuccessful = true;
		try {
			session = SessionFactory.newSession(serverURL, new UsernamePasswordCredentials(userid, password));
		} catch (AuthenticationException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (PasswordExpiredException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (UserInactiveException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (UserRetiredException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (RemoteException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (UnsupportedCredentialsException ex) {
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		}
		switch (command) {
			case 0: //test
				if (connectionSuccessful) {
					System.out.println("Connection successful!");
				} else {
					System.out.println("\n**Connection to " + serverURL + " failed.**\n");
				}
				break;
			case 1: //list
				if (session != null) {
					repositoryService = session.getServiceManager().getRepositoryService();
					try {
						List children = repositoryService.getChildren(new ContainerBean(listPath));
                        for (Iterator iter = children.iterator(); iter
                                .hasNext();) {
                            Node node = (Node) iter.next();
                            System.out.println(node.getPath());
                        }
					} catch (InvalidTypeException ex) {
						System.err.println(ex.getMessage());
					} catch (InvalidNodeException ex) {
						System.err.println(ex.getMessage());
					} catch (RemoteException ex) {
						System.err.println(ex.getMessage());
					} catch (InvalidSessionException ex) {
						System.err.println(ex.getMessage());
					}
				} else {
					System.err.println("Session not established.  Cannot execute command.");
				}
				break;
			default:
				showUsage();
				System.exit(1);
				break;
		}
			
		System.exit(0);
	}
	
	private static void showUsage()
	{
		System.out.println("Usage: SASDrugDevListSample [-s <server URL>] [-u <username>] [-p <password>] <command> [<command inputs>]");
		System.out.println("       where commands are:\n");
		System.out.println("           -test");
		System.out.println("           -list <SASDrugDev path>");
	}
	
	private static char[] readInput(String prompt)
	{
		char[] ret = new char[] {'\0'};
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			System.out.print(prompt);
			ret = (in.readLine()).toCharArray();
		} catch (IOException e) {
			System.err.println("Error reading input.");
		}
		return ret;
	}
	
}
